#! /usr/bin/env python

'''
Created on 15.05.2011

@author: Mehmet Ali Anil

The Monte-Carlo Simulation Engine
'''

import numpy as num
import pylab
from mpl_toolkits.mplot3d import Axes3D
import math
import matplotlib.pyplot as plt     # Plotting
import profile                      # For performance analysis


__author__ = "Mehmet Ali Anil, Ege Ozgun, Kivanc Esat, Deniz Gunceler"
 __copyright__ = "(C) 2011 Mehmet Ali Anil(mehmet.ali.anil@ieee.org)"
__credits__ = ["Mehmet Ali Anil", "Ege Ozgun", "Kivanc Esat", "Deniz Gunceler"]
__license__ = " "
__version__ = "0.0.1"
__maintainer__ = "Mehmet Ali Anil"
__email__ = "mehmet.ali.anil@ieee.org"
__status__ = "Production"



class atomic_lattice(object):
    """
    Atomic Lattice class for Monty Carlo simulations
        This object is meant to be a sole representative of a monatomic 
        lattice class, in which all atoms incorporated will have the same 
        structure.
    """
    def __init__(self,(vector_1,vector_2,vector_3),atoms_unit,strech=1):
        """
        Defines the lattice object
        lattice((vector_1,vector_2,vector_3), atoms, strech = 1)
                
                (vector_1,vector_2,vector_3)
            
            These are the translational symmetry vectors that the unit cell will 
            repeat itself. Their types can be a python list. 
        
            Example:
            
            ([1,0,0],[0,1,0],[0,0,1]) 
            
            will create a lattice with the unit cell is a cube, 
            and it repeats itself for all linear combinations
            of these unit vectors.
            
                atoms
            
            This is a Python list of the coordinates of the atoms with respect to 
            the coordinate system formed with (vector_1,vector_2,vector_3).
            
            An example may be:
            
            [[0,0,0],[0,0.5,0.5],[0.5,0.5,0],[0.5,0,0.5],[1,0.5,0.5],
            [0.5,0.5,1],[0.5,1,0.5],[1,0,0],[0,1,0],[0,0,1],
            [1,1,0],[1,0,1],[0,1,1],[1,1,1]]
            
            which is the list for all atoms in a face centered cubic structure.
        """
        
        self.explanation = None
        
        print "Initializing the unit cell.."
        self.vectors_n_norm = num.array([vector_1,vector_2,vector_3])
        #Normalizes the bases vector.
        self.vectors = [self.n_norm_vec/num.linalg.norm(self.n_norm_vec) 
                        for self.n_norm_vec in self.vectors_n_norm]
       
        self.atoms_unit = num.array(atoms_unit)
        self.atoms_cartesian_unit = num.array([num.dot(atom_position,self.vectors) for 
                           (atom_no,atom_position) in enumerate(self.atoms_unit)])

        self.atoms = num.array([0,0,0])
        
        # list of all vectors that are linear superpositions of basis vectors
        
        symmetries = num.array([[x_ctr,y_ctr,z_ctr]
                                for x_ctr in range(strech+1) 
                                for y_ctr in range(strech+1) 
                                for z_ctr in range(strech+1)])
                        
        print "  Done!"
        print "Streching the crystal.."
    
        for atom in self.atoms_cartesian_unit:
            for symmetry_vec in symmetries:
      
                newcomer  = atom + symmetry_vec
                self.atoms = num.vstack([self.atoms,newcomer])
        print "  Done!"
        
    def show(self,unit=False):
        """
        Visualizes the structure of the crystal with an interactive 3d 
        interface.
        
            unit = False:
            
            If true, will show the unit cell only.
            
        """
        figure = pylab.figure()
        axes = Axes3D(figure)
        if unit:
            axes.scatter(self.atoms_unit[:,0],self.atoms_unit[:,1],
                         self.atoms_unit[:,2])
        else:
            axes.scatter(self.atoms[:,0],self.atoms[:,1],self.atoms[:,2])
        plt.show()
        
    
class ising_2d(object):
    """   
    This is the object for the system definitions of the 2d ising spin model.
    """   
    def __init__(self):
        self.state = None
        self.complementary_obj = {'neigh':None}
        self.parameters = {'anti_symmetric_cont':None,'symmetric_cont':None}
        self.dependencies = {'beta' : num.arange(0.01,1,0.01)}
        self.output_variables = ('magnetization',None)
        self.user_parameters = None
        self.energy = array([])
    
    def generate(self,type='random'):
        """
        Generates and returns a spin configuration. 
        """
        if type == 'random':
            num.random.seed()
            self.state = ((num.random.randint(0,2,(n,n))*2)-1)
        
    def post_generate(self):
        """
        Generates the spin neighborhood matrix for the state
        """
        if self.state == None:
            print "Please generate the spin configurations first by calling"
            print "name_of_system.generate()"
       
        else:
            print "Generating the neighborhood matrix of the state..."
            
            neighbours = num.zeros((len(self.state),len(len.state[0]))).tolist()
        
            for (y,row) in enumerate(self.state):
                for (x,spin) in enumerate(row):
                    if (y == len(self.state)-1 and x != len(self.state[0])-1):
                        neighbours[y][x] = (y-1,x),(y,x-1),(y,x+1),(0,x)
                    elif (y != len(self.state)-1 and x == len(self.state[0])-1):
                        neighbours[y][x] = (y-1,x),(y,x-1),(y,0),(y+1,x)
                    elif (y == len(self.state)-1 and x == len(self.state[0])-1):
                        neighbours[y][x] = (y-1,x),(y,x-1),(y,0),(0,x)
                    else:
                        neighbours[y][x] = (y-1,x),(y,x-1),(y,x+1),(y+1,x)    
         
            self.complementary_obj['neigh'] = neighbours 
            print "    Done."

    def energy(self):
        """ Gives the energy of an 2D Ising state."""

        energy=0
        
        symmetric_cont= self.parameters['symmetric_cont']
        anti_symmetric_cont=  self.parameters['anti_symmetric_cont']
        spin_cont=  self.parameters['spin_cont']
        
        for (y,row) in enumerate(self.state):
            for (x,spin) in enumerate(row):
                energy = energy + spin_cont
                if (self.state[x][y] == self.state[x-1][y]):
                    energy = energy + symmetric_cont
                else: 
                    energy = energy + anti_symmetric_cont
                    
                if (self.state[x][y] == self.state[x][y-1]):
                    energy = energy + symmetric_cont
                else:
                    energy = energy + anti_symmetric_cont
        
        return energy

    def energy_difference(self):
        """Calculates the energy difference of a particular spin, if it had been reversed."""
        energy = 0
        for neighbour in state_neigh[ctr_y][ctr_x]:
            energy = energy + state[neighbour[0]][neighbour[1]]
        delta_E = 2*spin*energy

        
class system(object):
    """   
    This is the system object for the simulation.
    It incorporates objects that are affiliated with the simulation,
    and 
    """
    def __init__(self):
        self.state_def = None
        self.parameters = None
        self.t = 0
        self.user_parameters = None
        self.energy = array([])

        
    def compute_energy(self):
        """ Computes the energy of the state and appends it into the list"""
        self.energy.append(self.energy_def(self.state_indicator))   






#! /usr/bin/env python

import atomic as a
import numpy as num
import logging
logging.basicConfig(level=logging.DEBUG)


Na = a.Atom(atomic_no=11)
Cl = a.Atom(atomic_no=17)
Na.size=25
Cl.size=40

[Na2,Na3,Na4,Na5,Na6,Na7,Na8,Na9,Na10,Na11,Na12,Na13,Na14] = Na.copy(13)
[Cl2,Cl3,Cl4,Cl5,Cl6,Cl7,Cl8,Cl9,Cl10,Cl11,Cl12,Cl13] = Cl.copy(12)


Na.position=num.array([0,0,0])
Cl.position=num.array([0.5,1,0])
Na2.position=num.array([.5,.5,0])
Cl2.position=num.array([0,.5,0])
Na3.position=num.array([1,1,0])
Cl3.position=num.array([.5,0,0])
Na4.position=num.array([1,0,0])
Cl4.position=num.array([1,0.5,0])
Na5.position=num.array([0,1,0])

Cl5.position=num.array([0,0,.5])
Na6.position=num.array([0.5,1,.5])
Cl6.position=num.array([.5,.5,.5])
Na7.position=num.array([0,.5,.5])
Cl7.position=num.array([1,1,.5])
Na8.position=num.array([.5,0,.5])
Cl8.position=num.array([1,0,.5])
Na9.position=num.array([1,0.5,.5])
Cl9.position=num.array([0,1,.5])

Na10.position=num.array([0,0,1])
Cl10.position=num.array([0.5,1,1])
Na11.position=num.array([.5,.5,1])
Cl11.position=num.array([0,.5,1])
Na12.position=num.array([1,1,1])
Cl12.position=num.array([.5,0,1])
Na13.position=num.array([1,0,1])
Cl13.position=num.array([1,0.5,1])
Na14.position=num.array([0,1,0])

list_of_atoms = [Na,Na2,Na3,Na4,Na5,Na6,Na7,Na8,Na9,Na10,Na11,Na12,Na13,Na14,
                 Cl2,Cl3,Cl4,Cl5,Cl6,Cl7,Cl8,Cl9,Cl10,Cl11,Cl12,Cl13]

unit_vec = (num.array([1,0,0]),num.array([0,1,0]),num.array([0,0,1]))
lat = a.Lattice(unit_vec,list_of_atoms,strech=3)
lat.show()


#!/usr/bin/env python

from objectdefs import *
import matplotlib.pyplot as plt
import copy
import numpy as num
import pylab
from mpl_toolkits.mplot3d import Axes3D

print_en = True

def ising_interaction_func(element,surroundings,self_contrib=1,surr_contrib=1):
    energy = self_contrib * element.spin
    for other_object in surroundings:
        energy = energy + element.spin*other_object.spin*surr_contrib
        
ising_interaction = ElementalInteraction() 
ising_interaction.function = ising_interaction_func()

def ising_energy_def():
    pass


"""
Probable Class definition
"""
import numpy as num

class probeable_obj ():
    def __init__ (self):
        self.probes = []
    def attach (self,probe):
        if probe in self.probes:
            print "This probe is already attached."
        else:
            self.probes.append(probe)
    
    def populate_probes (self,subroutine):
        for probe in self.probes:
            if probe.subroutine == subroutine:
                measured = probe.function(self)
                print measured
                probe.data = num.append(probe.data,measured)
                


def init_parallel(servers=()):
    """
    Initializes a parallel python session
    """
    import sys
    import pp
    
    ppservers = servers
    if len(sys.argv) > 1:
        ncpus = int(sys.argv[1])
        # Creates jobserver with ncpus workers
        job_server = pp.Server(ncpus, ppservers=ppservers)
    else:
        # Creates jobserver with automatically detected number of workers
        job_server = pp.Server(ppservers=ppservers)
    
    if print_enable:
        print "Parallel Python is configured to work with", job_server.get_ncpus(), "workers"
    

def push_job(job):
    pass
        


'''
Created on Aug 12, 2011

@author: mali
'''

from objectdefs import *
import numpy as num
import pylab
from mpl_toolkits.mplot3d import Axes3D
import math
import matplotlib.pyplot as plt     # Plotting
import profile                      # For performance analysis


def machine(system):
    if type(system) != system:
        print "The Monte Carlo engine must have the input system of type system."
        print "Please prepare your state as an system type object."
    else:
        print "Initializing the algorithm..."
        
        
        # Checks are to be made with the other input parameters.
        magnetization_beta = []
        
        for dependency in system.dependencies:
            for dep_name in dependency:
                magnetization_list = num.array([])
                energy_list = num.array([])
                standard_dev = 0
                err=0
                ctr=0
                print "Simulating the system with beta = "+str(beta)
                while  (err > 20 or err == 0) or ctr<20:
                    (state,params) = monte_carlo_iter(neigh,state,beta,plot=False)
                    magnetization_list = num.append(magnetization_list,params[0])
                    energy_list = num.append(energy_list,params[1])
                    # We take the standard deviation of the last four  divided by the 
                    standard_dev = num.std(energy_list[-4:])
                    err = standard_dev/num.mean(energy_list[-4:])
                    print "    ["+str(ctr)+"]Standard Deviation: " + str(standard_dev)  
                    ctr = ctr + 1      
                magnetization_beta.append(magnetization_list[-1])


from __future__ import division


"""
Provides classes relating to MC statistics with atoms
"""

from objectdefs import *
import matplotlib.pyplot as plt
import copy
import numpy as num
import pylab
from mpl_toolkits.mplot3d import Axes3D

print_en = True

def print_en(selection):
    """
    Print enable function
    """
    global print_en
    if selection:
        print_en = True
    else:
        print_en = False
        
        
class Particle (Element):
    """
    Particle class, a representation of small particles
	"""
    def __init__(self, type='p', *args, **kwargs):
        self.spin = None
        self.energy = None
        self.type = type
    
    def copy(self):
        new_particle = copy.copy(self)
        return new_particle 
 
class Atom(Element):
    """
    Atom class, a representation of an atom.
    """
    def __init__(self, atomic_no = None, *args, **kwargs):
        
        # initialization of an atom class
        self.atomic_number = atomic_no
        self.mass_number = None
        self.charge = None
        self.magnetic_moment = None
        self.stable = True
        self.position = num.array([0,0,0],dtype=float)
        self.momentum = num.array([0,0,0],dtype=float)
        
        # Probability of decay per unit time 
        self.decay_prob = None
        
        # Decay process: [Change in proton number, neutron number, particle object] 
        self.decay_process = [None, None, None]
        
        # A pseudo radius for close packing
        self.size = None
        
        # COLOR
        if self.atomic_number == None:
            self.color = 'r'
        else:
            color_1 = (self.atomic_number % 7) / 6
            color_2 = ((self.atomic_number / 7) % 7) / 6
            color_3 = ((self.atomic_number / 7) / 7) / 6
            self.color = (color_1 ,color_2 ,color_3)
            
    def copy(self,n=1):
        """
        Copies an existing atom and returns a copy of it
        """
        atom_copy_list = [copy.copy(self) for n in range(n)]
        return atom_copy_list
    
    def reinit(self):
        """
        Derived properties of the atom is reinitalized
        """
        if (self.mass_number != None and self.atomic_number != None):
            self.neutron = self.mass_number - self.atomic_number
            self.proton = self.atomic_number
        else:
            print "The mass number or the atomic number is defined as null "
            
        # COLOR
                
        if self.atomic_number == None:
            self.color = 'r'
        else:
            color_1 = (self.atomic_number % 7) / 6
            color_2 = ((self.atomic_number / 7) % 7) / 6
            color_3 = ((self.atomic_number / 7) / 7) / 6
            self.color = (color_1 ,color_2 ,color_3)
        

    def decay(self):
        """The atom undergoes a nuclear decay, as defined in __init__
        if the atom.stable = False
        """
        if self.decay_prob == False:
            print "This atom is stable, it wont undergo any decay."
            print "Please define the atom as unstable first."
        else:
            print "Atom is undergoing decay."
            
            self.atomic_number = self.atomic_number - self.decay_process[0]
            self.mass_number = self.mass_number - (self.decay_process[1] +
                                                    self.decay_process[0])
            self.reinit()
            radiation = self.decay_process[2]
            
            return radiation
        
    def position(self,new_position):
        """
        Checks whether a valid position vector is defined for the atom and then 
        assigns it.
        """
        if type(new_position) == list:
            self.position = num.array(new_position).astype(float)
        else:
            print "There's something wrong with the assigned value for the"
            print "position vector. Please assign it as a list or a numpy array."
            print "e.g.  atom.position([0,0,0])"
            print "e.g.  atom.position(num.array([0,0,0]))"
            
class AtomicConfiguration(State):
    """
    Atomic Configuration is a looser Lattice Class
    in which the lattice symmetry is not mandatory.
    """
    def __init__(self,dim,*args,**kwargs):
        """
        Defines the Atomic Configuration Class
        """
        self.atoms = []
        self.dim = dim
        
    def copy(self):
        new_lattice = copy.copy(self)
        return new_lattice
        
        
class Lattice(State):
    """
    Atomic Lattice class for Monty Carlo simulations
        This object is meant to be a sole representative of a monatomic 
        lattice class, in which all atoms incorporated will have the same 
        structure.
    """
    def __init__(self,vectors,atoms_basis,strech=1, *args, **kwargs):
        """
        Defines the lattice object
        lattice((vector_1,vector_2,vector_3), atoms, strech = 1)
                
                (vector_1,vector_2,vector_3)
            
            These are the translational symmetry vectors that the unit cell will 
            repeat itself. Their types can be a Python list. 
        
            Example:
            
            ([1,0,0],[0,1,0],[0,0,1]) 
            
            will create a lattice with the unit cell is a cube, 
            and it repeats itself for all linear combinations
            of these unit vectors.
            
                atoms
            
            This is a list of atom objects, with defined coordinates.
        """
        
        self.explanation = None
        self.dim = len(vectors)
        
        if print_en:
            print "Initializing the unit cell.."
        
        self.unit_vectors_not_norm = num.array(vectors)
        
        # Normalizes the base vectors.
        self.unit_vectors = [self.n_norm_vec/num.linalg.norm(self.n_norm_vec) 
                             for self.n_norm_vec in self.unit_vectors_not_norm]
        
        # atoms_unit is a list of (num.array(x,y,z), atom)
        self.atoms_basis = atoms_basis
        self.atoms_unit = [(num.dot(atom.position,self.unit_vectors),atom) 
                                               for atom in self.atoms_basis]

        self.atoms = []
        
        # list of all vectors that are linear super positions of basis vectors
        if self.dim==3: 
            symmetries = num.array([[x_ctr,y_ctr,z_ctr]
                                    for x_ctr in range(strech) 
                                    for y_ctr in range(strech) 
                                    for z_ctr in range(strech)])
        elif self.dim==2:
            symmetries = num.array([[x_ctr,y_ctr]
                                    for x_ctr in range(strech) 
                                    for y_ctr in range(strech)])
        elif self.dim==1:
            symmetries = num.array([[x_ctr]
                                    for x_ctr in range(strech)])
            
        print symmetries
                        
        print "  Done!"
        print "Streching the crystal.."
    
        coordinates_only = []
        for (atom_coordinate, atom) in self.atoms_unit:
            for symmetry_vec in symmetries:
                
                newcomer_coordinate  = atom_coordinate + symmetry_vec
                [newcomer] = atom.copy()
                
                if any(((newcomer_coordinate == coor).all() 
                        for coor in coordinates_only)):
                    print "WARNING: Duplicate coordinate, newcomer ignored."
                else:
                    newcomer.coordiante = newcomer_coordinate
                    self.atoms.append(newcomer)
                    coordinates_only.append(newcomer_coordinate)
                
        print "  Done!"
        
    def strech(self,strech):
        """
        Streches the crystal from scratch
        """
        if print_en:
            print "Streching the crystal.."
        
        self.atoms = []
        
        # list of all vectors that are linear superpositions of basis vectors
        if self.dim==3: 
            symmetries = num.array([[x_ctr,y_ctr,z_ctr]
                                    for x_ctr in range(strech) 
                                    for y_ctr in range(strech) 
                                    for z_ctr in range(strech)])
        elif self.dim==2:
            symmetries = num.array([[x_ctr,y_ctr]
                                    for x_ctr in range(strech) 
                                    for y_ctr in range(strech)])
        elif self.dim==1:
            symmetries = num.array([[x_ctr]
                                    for x_ctr in range(strech)])
            
        print symmetries
        
        if print_en:                
            print "  Done!"
            print "Streching the crystal.."
    
        coordinates_only = []
        for (atom_coordinate, atom) in self.atoms_unit:
            for symmetry_vec in symmetries:
                
                newcomer_coordinate  = atom_coordinate + symmetry_vec
                [newcomer] = atom.copy()
                
                if any(((newcomer_coordinate == coor).all() 
                        for coor in coordinates_only)):
                    if print_en:
                        print "WARNING: Duplicate coordinate, newcomer ignored."
                else:
                    newcomer.coordinate = newcomer_coordinate
                    self.atoms.append(newcomer)
                    coordinates_only.append(newcomer_coordinate)
        if print_en:           
            print "  Done!"    
            
    def show(self,unit=False):
        """
        Visualizes the structure of the crystal with an interactive 3d 
        interface.
        
            unit = False:
            
            If true, will show the unit cell only.
            
        """
        if self.dim == 3:
            figure = pylab.figure()
            axes = Axes3D(figure)
                
            if unit and self.unit != None :
                for (atom_coor , atom) in self.atoms_unit:
                    if hasattr(atom, 'color') and hasattr(atom, 'size'):
                        plot_dict = {'c':atom.color,'s':atom.size}
                    elif  hasattr(atom, 'size'): 
                        plot_dict = {'s':atom.size}
                    elif  hasattr(atom, 'color'):
                        plot_dict = {'c':atom.color}
                    else:
                        plot_dict = {}
                        
                    axes.scatter3D(num.array([atom_coor[0]]),
                                 num.array([atom_coor[1]]),
                                 num.array([atom_coor[2]]),
                                 **plot_dict)
            elif not(unit):
                for atom in self.atoms:
                    if hasattr(atom, 'color') and hasattr(atom, 'size'):
                        plot_dict = {'c':atom.color,'s':atom.size}
                    elif  hasattr(atom, 'size'):
                        plot_dict = {'s':atom.size}
                    elif  hasattr(atom, 'color'):
                        plot_dict = {'c':atom.color}
                    else:
                        plot_dict = {}
                    
                    axes.scatter3D(num.array([atom.position[0]]),
                                 num.array([atom.position[1]]),
                                 num.array([atom.position[2]]),
                                 **plot_dict)
        elif self.dim < 3:
            figure = pylab.figure()
    
            if unit and self.unit != None:
                for (atom_coor , atom) in self.atoms_unit:
                    if hasattr(atom, 'color') and hasattr(atom, 'size'):
                        plot_dict = {'c':atom.color,'s':atom.size}
                    elif  hasattr(atom, 'size'):
                        plot_dict = {'s':atom.size}
                    elif  hasattr(atom, 'color'):
                        plot_dict = {'c':atom.color}
                    else:
                        plot_dict = {}
                    plt.scatter(num.array([atom_coor[0]]),
                                 num.array([atom_coor[1]]),
                                 **plot_dict)
            elif not(unit):
                for atom in self.atoms:
                    if hasattr(atom, 'color') and hasattr(atom, 'size'):
                        plot_dict = {'c':atom.color,'s':atom.size}
                    elif  hasattr(atom, 'size'):
                        plot_dict = {'s':atom.size}
                    elif  hasattr(atom, 'color'):
                        plot_dict = {'c':atom.color}
                    else:
                        plot_dict = {}
                    plt.scatter(num.array([atom.position[0]]),
                                 num.array([atom.position[1]]),
                                 **plot_dict)
        plt.show()
        
    def merge(self, lattice_prim , lattice_sec, offset = [0,0,0]):
        """
        Merges two crystals with an offset for the second one.
        """
        dim = max(len(lattice_prim.dim),len(lattice_sec.dim),len(offset))
        output_conf = AtomicConfiguration(dim)
        lattice_prim_newatoms= lattice_prim.deepcopy().atoms
        new_lattice = lattice_sec.deepcopy()
        for atom in new_lattice:
            atom.position = atom.position + offset
        lattice_sec_newatoms= new_lattice.atoms
        output_conf.atoms = lattice_prim_newatoms.append(lattice_sec_newatoms)
        return output_conf
    
    def copy(self):
        new_lattice = copy.copy(self)
        return new_lattice
    



"""
Object definitions for a general system in concern. 
"""

import probes

class Element(object,probes.probeable_obj):
    """
    General definition of an element object.
    """
    def __init__(self, *args, **kwargs):
        probes.probeable_obj.__init__(self)
        self.explanation = ""
        self.parameters = []

class State(object,probes.probeable_obj):
    """
    Definition of the state object.
    """
    def __init__(self, *args, **kwargs):
        probes.probeable_obj.__init__(self)
        self.explanation = ""
        self.parameters = []
        self.constituents = None
        
class Action(object,probes.probeable_obj):
    """
    Definition of the action object.
    Encapsulates an action on a state.
    """
    def __init__(self, *args, **kwargs):
        probes.probeable_obj.__init__(self)
        self.explanation = ""
        self.actson_states = None
        self.actson_elements = None 
        self.language = None
        self.function = None
        
class System(object,probes.probeable_obj):
    """
    Definition of the system object.
    """
    def __init__(self, *args, **kwargs):
        probes.probeable_obj.__init__(self)
        self.explanation = ""

class Potential (object,probes.probeable_obj):
    """
    A potential class.
    Defined with the presence of two elements of the system.
    """
    def __init__(self):
        probes.probeable_obj.__init__(self)
        self.explanation = ""
        
class ElementalInteraction (object,probes.probeable_obj):
    """
    A class for every interaction concerning two element-type objects
    """
    def __init__(self,function, *args, **kwargs):
        """
        an object of ElementalInteraction is defined with function that has
        two objects of type element as input.
        """
        probes.probeable_obj.__init__(self)
        self.function = function
        self.explanation = "" 
        
        
        
class EnergyDefinition (object,probes.probeable_obj):
    """
    Class for energy definitions.
    An enery definition needs an elemental interaction
    And a rule of whom are interacting.
    """
    def __init__(self, elem_inter , rule, *args, **kwargs):
        probes.probeable_obj.__init__(self)
        pass


#! /usr/bin/env python

import atomic as a
import numpy as num
import logging
logging.basicConfig(level=logging.DEBUG)


Na = a.Atom(atomic_no=11)
Cl = a.Atom(atomic_no=17)
Na.size=25
Cl.size=40

[Na2] = Na.copy(1)
[Cl2] = Cl.copy(1)


Na.position=num.array([0,0])
Cl.position=num.array([.5,0])
Na2.position=num.array([.5,.5])
Cl2.position=num.array([0,.5])


list_of_atoms = [Na,Na2,Cl,Cl2]

unit_vec = (num.array([1,0]),num.array([0,1]))
lat = a.Lattice(unit_vec,list_of_atoms,strech=3)
lat.show()


#! /usr/bin/env python



import numpy as num
import math
import matplotlib.pyplot as plt     # Plotting
import profile                      # For performance analysis
import pp                           # For parallelization
import time
job_server = pp.Server() 


__author__ = "Mehmet Ali Anil"
 __copyright__ = "(C) 2011 Mehmet Ali Anil(mehmet.ali.anil@ieee.org)"
__credits__ = ["Mehmet Ali Anil"]
__license__ = " "
__version__ = "0.0.1"
__maintainer__ = "Mehmet Ali Anil"
__email__ = "mehmet.ali.anil@ieee.org"
__status__ = "Production"

def energy_ising_3d(state, type="closest", param={'symmetric_cont':-1, 
    'anti_symmetric_cont':1,'spin_cont':1}):
    """ Gives the energy of an 2D Ising state."""
    
    if (type == "closest"):
        energy=0
        
        symmetric_cont= param['symmetric_cont']
        anti_symmetric_cont=  param['anti_symmetric_cont']
        spin_cont=  param['spin_cont']
        
        for (z,slice) in enumerate(state):
            for (y,row) in enumerate(slice): 
                for (x,spin) in enumerate(row):
                    energy = energy + spin_cont
                    if (state[z][y][x] == state[z-1][y][x]):
                        energy = energy + symmetric_cont
                    else: 
                        energy = energy + anti_symmetric_cont
                        
                    if (state[z][y][x] == state[z][y-1][x]):
                        energy = energy + symmetric_cont
                    else:
                        energy = energy + anti_symmetric_cont
                                                
                    if (state[z][y][x] == state[z][y][x-1]):
                        energy = energy + symmetric_cont
                    else:
                        energy = energy + anti_symmetric_cont
                        
        return  energy


def energy_ising_2d(state, type="closest", param={'symmetric_cont':-1, 
    'anti_symmetric_cont':1,'spin_cont':1}):
    """ Gives the energy of an 2D Ising state."""
    
    if (type == "closest"):
        energy=0
        
        symmetric_cont= param['symmetric_cont']
        anti_symmetric_cont=  param['anti_symmetric_cont']
        spin_cont=  param['spin_cont']
        
        for (y,row) in enumerate(state):
            for (x,spin) in enumerate(row):
                energy = energy + spin_cont
                if (state[x][y] == state[x-1][y]):
                    energy = energy + symmetric_cont
                else: 
                    energy = energy + anti_symmetric_cont
                    
                if (state[x][y] == state[x][y-1]):
                    energy = energy + symmetric_cont
                else:
                    energy = energy + anti_symmetric_cont
    return  energy

def neighbour_preallocation_ising(state):
    """
    http://stackoverflow.com/questions/6841206/
    creating-a-new-list-from-elements-of-an-
    other-list-referencing-the-elements-of-t
    
    
    """
    neighbours = num.zeros((len(state),len(state[0]))).tolist()
    
    for (y,row) in enumerate(state):
        for (x,spin) in enumerate(row):
            if (y == len(state)-1 and x != len(state[0])-1):
                neighbours[y][x] = (y-1,x),(y,x-1),(y,x+1),(0,x)
            elif (y != len(state)-1 and x == len(state[0])-1):
                neighbours[y][x] = (y-1,x),(y,x-1),(y,0),(y+1,x)
            elif (y == len(state)-1 and x == len(state[0])-1):
                neighbours[y][x] = (y-1,x),(y,x-1),(y,0),(0,x)
            else:
                neighbours[y][x] = (y-1,x),(y,x-1),(y,x+1),(y+1,x)    
 
    return neighbours


def neighbour_preallocation_ising_3d(state):
    """
    http://stackoverflow.com/questions/6841206/
    creating-a-new-list-from-elements-of-an-
    other-list-referencing-the-elements-of-t
    """
    
    neighbours = num.zeros((len(state),len(state[0]),len(state[0][0]))).tolist()
    
    for (z,slice) in enumerate(state):
        for (y,row) in enumerate(slice):
            for (x,spin) in enumerate(row):
                if (z == len(state)-1 and y == len(state[0])-1 and x != len(state[0][0])-1):
                    # zy
                    neighbours[z][y][x] = (z,y-1,x),(z,y,x-1),(z,y,x+1),(z,0,x),(z-1,y,x),(0,y,x)
                elif (z == len(state)-1 and y != len(state[0])-1 and x == len(state[0][0])-1):
                    # zx
                    neighbours[z][y][x] = (z,y-1,x),(z,y,x-1),(z,y,0),(z,y+1,x),(z-1,y,x),(0,y,x)
                elif (z == len(state)-1 and y == len(state[0])-1 and x == len(state[0][0])-1):
                    # zyx
                    neighbours[z][y][x] = (z,y-1,x),(z,y,x-1),(z,y,0),(z,0,x),(z-1,y,x),(0,y,x)
                elif (z != len(state)-1 and y == len(state[0])-1 and x != len(state[0][0])-1):
                    # y
                    neighbours[z][y][x] = (z,y-1,x),(z,y,x-1),(z,y,x+1),(z,0,x),(z-1,y,x),(z+1,y,x)
                elif (z != len(state)-1 and y != len(state[0])-1 and x == len(state[0][0])-1):
                    # x
                    neighbours[z][y][x] = (z,y-1,x),(z,y,x-1),(z,y,0),(z,y+1,x),(z-1,y,x),(z+1,y,x)
                elif (z == len(state)-1 and y != len(state[0])-1 and x != len(state[0][0])-1):
                    # z
                    neighbours[z][y][x] = (z,y-1,x),(z,y,x-1),(z,y,0),(z,y+1,x),(z-1,y,x),(0,y,x)
                elif (z != len(state)-1 and y == len(state[0])-1 and x == len(state[0][0])-1):
                    # yx
                    neighbours[z][y][x] = (z,y-1,x),(z,y,x-1),(z,y,0),(z,0,x),(z-1,y,x),(0,y,x)
                else:
                    neighbours[z][y][x] = (z,y-1,x),(z,y,x-1),(z,y,x+1),(z,y+1,x),(z-1,y,x),(z+1,y,x)
     
    return neighbours


def generate_random(n):
    '''
    Generates and returns a spin configuration. 
    '''
    
    num.random.seed()
    random_booleans = ((num.random.randint(0,2,(n,n))*2)-1)
   
    return random_booleans

def generate_random_3d(n):
    '''
    Generates and returns a 3d spin configuration. 
    '''
    
    num.random.seed()
    random_booleans = ((num.random.randint(0,2,(n,n,n))*2)-1)
   
    return random_booleans

def monte_carlo_iter(state_neigh,state,beta,plot=False,return_energy=True,
                     return_magnetization=True):
    for (ctr_y,row) in enumerate(state):
        for (ctr_x,spin) in enumerate(row):
            energy = 0
            for neighbour in state_neigh[ctr_y][ctr_x]:
                energy = energy + state[neighbour[0]][neighbour[1]]
            delta_E = 2*spin*energy
            random_num = num.random.rand()
            if random_num < math.exp(-beta*delta_E):
                state[ctr_y][ctr_x] = -state[ctr_y][ctr_x]
    if return_energy:                            
        energy = energy_ising_2d(state)
    else: 
        energy = None
    if return_magnetization:
        magnetization = state.sum()
    else:
        magnetization = None
    
    if plot :
        plot_state(state)
        time.sleep(0.05)
    return state,(magnetization,energy)


def monte_carlo_iter_3d(state_neigh,state,beta,plot=False,return_energy=True,
                     return_magnetization=True):
    for (ctr_z,slice) in enumerate(state):    
        for (ctr_y,row) in enumerate(slice):
            for (ctr_x,spin) in enumerate(row):
                energy = 0
                for neighbour in state_neigh[ctr_z][ctr_y][ctr_x]:
                    energy = energy + state[neighbour[0]][neighbour[1]][neighbour[2]]
                delta_E = 2*spin*energy
                random_num = num.random.rand()
                if random_num < math.exp(-beta*delta_E):
                    state[ctr_z][ctr_y][ctr_x] = -state[ctr_z][ctr_y][ctr_x]
    if return_energy:                            
        energy = energy_ising_3d(state)
    else: 
        energy = None
    if return_magnetization:
        magnetization = state.sum()
    else:
        magnetization = None
    
    if plot :
        plot_state(state)
        time.sleep(0.05)
        
    return state,(magnetization,energy)


def plot_state(state):
    plt.imshow(state,cmap=plt.cm.binary,interpolation='nearest')
    plt.show()
    plt.draw()
    

def monte_carlo_2d():
    state = generate_random(20)
    neigh = neighbour_preallocation_ising(state)
    magnetization_beta = []
    for beta in num.arange(0.02,1,0.02):
        magnetization_list = num.array([])
        energy_list = num.array([])
        standard_dev = 0
        err=0
        ctr=0
        print "Simulating the system with beta = "+str(beta)
        while  (err > 0.1 or err == 0) or ctr<40:
            (state,params) = monte_carlo_iter(neigh,state,beta,plot=False)
            magnetization_list = num.append(magnetization_list,params[0])
            energy_list = num.append(energy_list,params[1])
            standard_dev = num.std(num.abs(magnetization_list[-4:]))
            err = standard_dev/num.mean(num.abs(magnetization_list[-4:]))
            ctr = ctr + 1      
        print "    ["+str(ctr)+"]Error: " + str(err)
        magnetization_beta.append(num.mean(magnetization_list[-4:]))
    return magnetization_beta
        
        
def monte_carlo_3d():
    state = generate_random_3d(15)
    neigh = neighbour_preallocation_ising_3d(state)
    magnetization_beta = []
    for beta in num.arange(0.01,1,0.01):
        magnetization_list = num.array([])
        energy_list = num.array([])
        standard_dev = 0 
        ctr=0
        print "Simulating the system with beta = "+str(beta)
        while  (standard_dev > 25 or standard_dev == 0) or ctr<20:
            (state,params) = monte_carlo_iter_3d(neigh,state,beta,plot=False)
            magnetization_list = num.append(magnetization_list,params[0])
            energy_list = num.append(energy_list,params[1])
            standard_dev = num.std(energy_list[-4:])
            print "    ["+str(ctr)+"]Standard Deviation: " + str(standard_dev)  
            ctr = ctr + 1      
        magnetization_beta.append(magnetization_list[-1])
    return(state,magnetization_beta)
    


import numpy as num 

class SubRoutine(object):
    def __init__ (self):
        pass
        
    
monte_carlo = SubRoutine()
dice_throw = SubRoutine()


class probe (object):
    """
    Definition of the probe 
    A probe is has an assignee, and it is defined by listing a probe
    in the list "probes" within that object. (it can be a family or
    a network, obviously)
    The function is defined due to the Network 
    """
    def __init__ (self):
        self.function = None
        self.subroutine = None
        self.data = num.array([])

class mean_score_probe (probe):
    def __init__ (self):       
        probe.__init__(self)
        self.function = self.mean_score
        self.subroutine = dice_throw
        
    def mean_score(self,probeable):
        return num.mean(probeable.scores)
    
class eq_score_probe (probe):
    def __init__ (self):       
        probe.__init__(self)
        self.function = self.eq_score
        self.subroutine = monte_carlo
        
    def eq_score(self,probeable):
        print num.mean(probeable.equilibria)
        return num.mean(probeable.equilibria)


    
    


#! /usr/bin/env python

'''
Created on 15.05.2011

@author: Mehmet Ali Anil


'''

import numpy as num
import energy_def_lib as mc
import profile 

#import psyco ; psyco.jit() 
#from psyco.classes import *

__author__ = "Mehmet Ali Anil"
 __copyright__ = "(C) 2011 Mehmet Ali Anil(mehmet.ali.anil@ieee.org)"
__credits__ = ["Mehmet Ali Anil"]
__license__ = " "
__version__ = "0.0.1"
__maintainer__ = "Mehmet Ali Anil"
__email__ = "mehmet.ali.anil@ieee.org"
__status__ = "Production"

if __name__ == '__main__':
    # If performance analysis will be made.
    #profile.run('print mc.monte_carlo_2d(); print')


from setuptools import setup, find_packages

setup(
    name = "Monty Carlo",
    version = "0.1.1 dev",
    packages = find_packages(),
    # scripts = ['say_hello.py'],

    # Project uses reStructuredText, so ensure that the docutils get
    # installed or upgraded on the target machine
    install_requires = ['docutils>=0.3'],

    package_data = {
        # If any package contains *.txt or *.rst files, include them:
        '': ['*.txt', '*.rst'],
        # And include any *.msg files found in the 'hello' package, too:
        'hello': ['*.msg'],
    },

    # metadata for upload to PyPI
    author = "Mehmet Ali Anil",
    author_email = "mehmet.ali.anil@ieee.org",
    description = "Monty Carlo is a modular Monte Carlo engine, suitable for statistical systems.",
    license = "LICENSE.txt",
    keywords = "monte carlo simulation statistical physics",
    url = "http://github.com/monty-carlo/monty-carlo/",    
    # project home page, if any

    classifiers=[
          'Development Status :: 4 - Beta',
          'Environment :: Console',
	  'Intended Audience :: Science/Research',
          'License :: Other/Proprietary License',
	  'Natural Language :: English',
          'Operating System :: MacOS :: MacOS X',
          'Operating System :: Microsoft :: Windows',
          'Operating System :: POSIX',
          'Programming Language :: Python :: 2.7',
          'Topic :: Scientific/Engineering :: Bio-Informatics',
          'Topic :: Scientific/Engineering :: Information Analysis',
          'Topic :: Scientific/Engineering :: Physics'
          ],
)



'''
Created on Aug 12, 2011

@author: mali, ege
'''

import numpy as num
import datetime
import profile



#class MyClass(object):
#    '''
#    classdocs
#    '''


def random_number(k):       # for the ideal case one should choose k as large as 
                            # possible lets say 12 at least, since we will have  
                            # 2**(k-2) numbers between [0,1], inreasing k exponentially 
                            # increases our sample of random numbers
    """
    generates a uniform list consisting of 
    2**(k-2) numbers in random order in the
    interval [0,1] 

    our recursion is in the following form:
    x_n+1=x_n * rho (mod 2**k)
    we work in base 2 and we drop the last
    2 digits of each element(again in base 
    2) and then convert it to decimal, so
    we will have each number from 0 to 
    2**(k-2) once in a shuffled order then
    by dividing each element by 2**(k-2)
    we will have a uniform list in the
    interval [0,1] 

    (since it is lengthy and harder to go
    from binary <--> decimal couple of
    times in this code we extend the same
    logic such that we can use the same algorithm
    using the decimal representation only, details
    can be found in the comments)

    when rho=8t-3 our list is 
    guaranteed to have a period of 2**(k-2)
        
    """
    
    now = datetime.datetime.now()
    
    x = num.zeros(2**(k-2),num.float64)
    rand_num = num.zeros(2**(k-2),num.float64)
    rand_num_normal = num.zeros(2**(k-2),num.float64)
    rand_num_bin = '0'  
    t = now.second    
#    ms = now.microsecond
    x[0] = (2*t+3) % (2**k)
    

    if t != 0:                   # to aviod rho being negative 
        rho = 8*t-3
    else:
        rho = 3
#    print rho
    for n in range(2**(k-2)-1):
        x[n+1] = (x[n] * rho) % (2**k)
#        print x
    for n in range(2**(k-2)):
        rand_num[n] = ( x[n] - ( x[n] % 4 ) ) / 4           # since all different x values are seperated
        rand_num_normal[n] = rand_num[n] / (2**(k-2) -1)    # by an interval of 4, by this procedure
                                                            # we generate all possible numbers between
    return  rand_num_normal #,rand_num                      # [0,2**(k-2)) then by normalizing it
                                                            # we obtain the uniform set of numbers
                                                            # in the interval [0,1]
                
#        '''
#        Constructor
#        '''

# Random Generator Trial

                                # it's hard to see the result in the command window 
                                # for k=12 so for the sake of illusturation k is
                                # chosen to be 10 here
def bigloop():
    randoms = []
    for stack in range(100):
        randoms = num.append(randoms,random_number(15))
        print stack
    return randoms


#profile.run('print bigloop(); print')

